/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.data;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.pua.prot.util.ExpressionResult;

import java.text.NumberFormat;

/**
 * Skalierte Ansicht auf ein Datenobjekt. Erbt von ProtocolDataUnscaledValue und überschreibt die relevanten Methoden.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
class ProtocolDataScaledValue extends ProtocolDataUnscaledValue {

	/**
	 * Skalierte Ansicht auf ein Datum
	 * @param attributeProtocolData Datum
	 */
	public ProtocolDataScaledValue(final AttributeProtocolData attributeProtocolData) {
		super(attributeProtocolData);
	}

	@Override
	public long longValue() {
		return getScaledLongValue();
	}

	@Override
	public double doubleValue() {
		return getScaledDoubleValue();
	}

	@Override
	public String getValueText() {
		try {
			final IntegerValueState valueState = getState();
			if(valueState != null) return valueState.getName();
			if(getScaling() == 1.0 && _data.getInternalValue().getType() == ExpressionResult.ResultType.LONG) return Long.toString(longValue());
			return _doubleNumberFormat.format(doubleValue());
		}
		catch(NumberFormatException ignored) {
			return _data.getInternalValue().toString();
		}
	}
}
