/*
 * Copyright 2015 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.util;

import de.bsvrz.pua.prot.exceptions.CriticalParserException;
import de.bsvrz.pua.prot.functions.ExpressionInterface;
import de.bsvrz.pua.prot.util.attributes.AttributeDescription;

import java.util.Arrays;

/**
 * Klasse um ExpressionTrees komfortabel zu erzeugen. Idealerweise statisch importieren
 *
 * @author Kappich Systemberatung
 */
public class ExpressionTreeBuilder {
	
	public static ExpressionTree literal(boolean b){
		return new ExpressionLeaf(b);
	}	
	
	public static ExpressionTree literal(String s){
		return new ExpressionLeaf(s);
	}	
	
	public static ExpressionTree literal(long l){
		return new ExpressionLeaf(l);
	}
		
	public static ExpressionTree literal(double d){
		return new ExpressionLeaf(d);
	}	
	
	public static ExpressionTree function(Class<? extends ExpressionInterface> function, ExpressionTree... parameters){
		try {
			return new ExpressionLeaf(function.getName(), Arrays.asList(parameters));
		}
		catch(CriticalParserException e) {
			throw new AssertionError(e);
		}
	}

	public static ExpressionTree attribute(AttributeDescription... description){
		return new ExpressionLeaf(description);
	}

}
