/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.functions;

import antlr.collections.AST;
import de.bsvrz.pua.prot.exceptions.SemanticErrorException;
import de.bsvrz.pua.prot.processing.util.BaseDataSet;
import de.bsvrz.pua.prot.processing.util.ExpressionResultAndState;
import de.bsvrz.pua.prot.processing.util.TempAttribut;
import de.bsvrz.pua.prot.util.*;
import de.bsvrz.pua.prot.util.attributes.RealElement;
import de.bsvrz.pua.prot.util.attributes.TempAttributeDescription;

import java.util.*;


/**
 * Maxiumum Funktion. Zur Bestimmung des Maximalwerts.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Max implements ExpressionInterface {

	public ExpressionResult evaluate(
			List<ExpressionTree> expressionList,
			Hashtable<RealElement, BaseDataSet> realElements,
			Hashtable<TempAttributeDescription, TempAttribut> tempElements, final int pseudoObjectIndex) {
		if(expressionList == null || expressionList.isEmpty()) return ExpressionResult.error(ErrorMessageBuilder.illegalArgumentCount(this, expressionList, true, ErrorMessageBuilder.NUMERIC));

		ExpressionResultAndState result = expressionList.get(0).evaluateEx(realElements, tempElements, pseudoObjectIndex);

		if(!result.getResult().isArithmetic()){
			return ExpressionResult.error(ErrorMessageBuilder.illegalArgument(this, expressionList, 0, result, ErrorMessageBuilder.NUMERIC));
		}
		
		ArithmeticOperationEx calc = new ArithmeticOperationEx(result);

		ExpressionResultAndState temp;
		ExpressionTree expression;

		for(int i = 1; i < expressionList.size(); i++) {
			expression = expressionList.get(i);
			temp = expression.evaluateEx(realElements, tempElements, pseudoObjectIndex);
			if(!temp.getResult().isArithmetic()){
				return ExpressionResult.error(ErrorMessageBuilder.illegalArgument(this, expressionList, i, temp, ErrorMessageBuilder.NUMERIC));
			}
			calc.set(temp);
			ExpressionResultAndState less = calc.less();
			if(less.getResult().getType() == ExpressionResult.ResultType.ERROR){
				result = less;
				calc.setLeft(result);
			}
			else if(less.getResult().getBool()) {
				result = temp;
				calc.setLeft(result);
			}
		}
		return result.getResult();
	}

	public ExpressionResult.ResultType getResultType(final List<ExpressionTree> expressionList, final AST ast) throws SemanticErrorException {
		if(expressionList == null || expressionList.isEmpty()) return ExpressionResult.ResultType.ERROR;
		ExpressionTree xTree = expressionList.get(0);
		final ExpressionResult.ResultType type = xTree.getResultType(ast);
		return type;
	}

	@Override
	public String getFunctionName() {
		return "max";
	}
}
